/*  Volumetric Sunlights aka GodRays aka Sunshafts
	made by K.D. (OGSE team)
	12.11.2012
	used papers:
	1) GPU Gems 3. Volumetric Light Scattering as a Post-Process (Kenny Mitchell)
	2)    .  2 -  -. (  Nikola Tesla)
	3) Crysis shaders :)*/

#ifndef	OGSE_SUNSHAFTS_H
	#define OGSE_SUNSHAFTS_H
	#include "common.h"
	#ifndef SS_QUALITY
		#define	SS_MAX_SAMPLES int(1)
	#else
		#if SS_QUALITY==1
			#define	SS_MAX_SAMPLES int(20)
		#else
			#if SS_QUALITY==2
				#define	SS_MAX_SAMPLES int(50)
			#else
				#if SS_QUALITY==3
					#define	SS_MAX_SAMPLES int(100)
				#endif
			#endif
		#endif
	#endif
	#define SS_FARPLANE float(180.0)
	uniform float4 sunshafts_params;	// x - exposure
	float normalize_depth(float depth)
	{
		return (saturate(depth/SS_FARPLANE));
	}
	 float3 blend_soft_light(float3 a, float3 b)
	{
		float3 c = 2 * a * b + a * a * (1 - 2 * b);
		float3 d = sqrt(a) * (2 * b - 1) + 2 * a * (1 - b);
		return ( b < 0.5 )? c : d;
	}
/*
	float random (float2 tc0)
	{
		float2 tc = tc0*24;
	tc += abs(sin(timers.y/10))*float2(0, 1);
	if (tc.y>1)
		tc.y -= 1;
	tc *= (sin(timers.y/10)*0.5+0.5);//*float2(0, 1);
	float4 color = tex2D(s_smap_depth, frac(tc));
	return (color.x*0.5+0.5);
	}
	*/
	float3 sunshafts(float2 tc, float3 init_color)
	{  
		// dist to the sun
		float sun_dist = SS_FARPLANE / (sqrt(1 - L_sun_dir_w.y * L_sun_dir_w.y));
		// sun pos
		float3 sun_pos_world = sun_dist*L_sun_dir_w + eye_position;
		float4 sun_pos_projected = mul(m_VP, float4(sun_pos_world, 1));
		float4 sun_pos_screen = convert_to_screen_space(sun_pos_projected)/sun_pos_projected.w;
		// sun-pixel vector
		float2 sun_vec_screen = sun_pos_screen.xy - tc;
		// calculate filtering effect. sunshafts must appear only when looking in the sun dir
		float angle_cos = dot(-eye_direction, normalize(L_sun_dir_w));
		float ray_fade = saturate(saturate(angle_cos)*(1 - saturate(dot(sun_vec_screen, sun_vec_screen)))*saturate(length(sun_vec_screen.xy)));
		float2 init_tc = tc;
		
		float depth = normalize_depth(tex2D(s_position, tc).z);
		float mask = (1.00001 - (1 - depth + 1 - saturate(saturate(angle_cos*0.1 + 0.9)))) * SS_BLEND_FACTOR;
		// adjust sampling
		sun_vec_screen *= angle_cos * SS_DENSITY / SS_MAX_SAMPLES;

		// sampling image along ray
		float3 accum = 0;
		for (int i = 0; i < SS_MAX_SAMPLES; i++)
		{ 
			tc += sun_vec_screen;
			depth = normalize_depth(tex2D(s_position, tc).z);
			accum += depth * (1 - i / SS_MAX_SAMPLES);
		}
		
		// applying filtering effect
		float4 sun_pos_end = tex2D(s_position, sun_pos_screen);
		// watch some magic: if sun is occluded by terrain or some buildings, sunshafts must be eliminated. Otherwise image is terrible.
/*		int mat;
		if (sun_pos_end.w > 0.1 && sun_pos_end.w < 0.5) {
			mat = 1;
		} else {
			mat = 1;
		};*/
		// try to calc some "noise"
//		float noise_factor = saturate(SS_NOISE + random(tc)*(sin(timers.y/10)*0.5+0.5)*(1-SS_NOISE));
		// adjusting assum effect. If sun is _not_ on sky (occluded by smthng), accum must be zero.
		accum *= 2 * ray_fade / SS_MAX_SAMPLES ;//* noise_factor;//saturate(1 - normalize_depth(sun_pos_end.z) * mat * 1000) * noise_factor;
		// blending
		init_color += accum * L_sun_color * (1.0 - init_color) * sunshafts_params.x;
		return blend_soft_light(init_color, L_sun_color * mask * 0.5 + 0.5);
	}  
	float3 calc_sunshafts(float2 tc, float3 init_color) {
		#ifndef	SS_QUALITY
			return init_color;
		#else
			return sunshafts(tc, init_color);
		#endif	
	}
#endif